==> libggi_1.99.2.0b2.1.orig.tar.gz: libggi-2.0b2.1/display/sub/stubs.c <==
/* $Id: stubs.c,v 1.14 1999/01/28 23:21:37 marcus Exp $
******************************************************************************

   Display-sub: stubs

   Copyright (C) 1998 Andreas Beck    [becka@ggi-project.org]
   Copyright (C) 1998 Rudolphe Ortalo

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

/* FIXME: Should we adjust parent clipping for "get" calls ? */

#include <ggi/display/sub.h>

#define IS_SAVING_GC	ggi_pixel _foreground, _background; \
			int _l,_t,_r,_b;

#define SETUP_AND_SAVE_GC \
		do {	_foreground = LIBGGI_GC(hook->parent)->fg_color; \
			_background = LIBGGI_GC(hook->parent)->bg_color; \
			_l = LIBGGI_GC(hook->parent)->cliptl.x; \
			_t = LIBGGI_GC(hook->parent)->cliptl.y; \
			_r = LIBGGI_GC(hook->parent)->clipbr.x; \
			_b = LIBGGI_GC(hook->parent)->clipbr.y; \
			LIBGGI_GC(hook->parent)->fg_color = LIBGGI_GC(vis)->fg_color; \
			LIBGGI_GC(hook->parent)->bg_color = LIBGGI_GC(vis)->bg_color; \
			LIBGGI_GC(hook->parent)->cliptl.x=LIBGGI_GC(vis)->cliptl.x+hook->position.x; \
		   	LIBGGI_GC(hook->parent)->cliptl.y=LIBGGI_GC(vis)->cliptl.y+hook->position.y; \
		   	LIBGGI_GC(hook->parent)->clipbr.x=LIBGGI_GC(vis)->clipbr.x+hook->position.x; \
		   	LIBGGI_GC(hook->parent)->clipbr.y=LIBGGI_GC(vis)->clipbr.y+hook->position.y; \
			/* check limits... (in case leaving visual) */ \
			if (LIBGGI_GC(hook->parent)->clipbr.x > hook->botright.x) \
				LIBGGI_GC(hook->parent)->clipbr.x = hook->botright.x; \
			if (LIBGGI_GC(hook->parent)->clipbr.y > hook->botright.y) \
				LIBGGI_GC(hook->parent)->clipbr.y = hook->botright.y; \
			/* ensure update in KGI */ \
			LIBGGI_GC(hook->parent)->version++; \
		} while (0);

#define RESTORE_GC \
		do {	LIBGGI_GC(hook->parent)->fg_color = _foreground; \
			LIBGGI_GC(hook->parent)->bg_color = _background; \
			LIBGGI_GC(hook->parent)->cliptl.x= _l; \
		   	LIBGGI_GC(hook->parent)->cliptl.y= _t; \
		   	LIBGGI_GC(hook->parent)->clipbr.x= _r; \
		   	LIBGGI_GC(hook->parent)->clipbr.y= _b; \
			/* ensure update in KGI */ \
			LIBGGI_GC(hook->parent)->version++; \
		} while (0);


/*
**	Functions undergoing a translation
**	with GC saving/restoring
*/

/* NB: Here the src is not directly translated... */
int GGI_sub_crossblit(ggi_visual *src, int sx, int sy, int w, int h,
		      ggi_visual *vis, int dx, int dy)
{
	SubHook *hook=LIBGGI_PRIVATE(vis); /* Destination is subvisual */
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiCrossBlit(src, sx, sy, w, h, hook->parent,
		      dx+hook->position.x, dy+hook->position.y);
	RESTORE_GC;
	return err;
}

int GGI_sub_drawbox(ggi_visual *vis,int x,int y,int width,int length)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawBox(hook->parent,x+hook->position.x,y+hook->position.y,width,length);
	RESTORE_GC;
	return err;
}

int GGI_sub_putbox(ggi_visual *vis,int x,int y,int width,int length,void *buffer)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPutBox(hook->parent,x+hook->position.x,y+hook->position.y,width,length,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_getbox(ggi_visual *vis,int x,int y,int width,int length,void *buffer)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiGetBox(hook->parent,x+hook->position.x,y+hook->position.y,width,length,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_fillscreen(ggi_visual *vis)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawBox(hook->parent, hook->position.x,hook->position.y,
					hook->botright.x-hook->position.x,
					hook->botright.y-hook->position.y);
	RESTORE_GC;
	return err;
}

int GGI_sub_putc(ggi_visual *vis,int x,int y,char c)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPutc(hook->parent,x+hook->position.x,y+hook->position.y,c);
	RESTORE_GC;
	return err;
}


int GGI_sub_puts(ggi_visual *vis,int x,int y,const char *str)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPuts(hook->parent,x+hook->position.x,y+hook->position.y,str);
	RESTORE_GC;
	return err;
}

int GGI_sub_drawhline(ggi_visual *vis,int x,int y,int w)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawHLine(hook->parent,x+hook->position.x,y+hook->position.y,w);
	RESTORE_GC;
	return err;
}

int GGI_sub_drawline(ggi_visual *vis,int x1,int y1,int x2,int y2)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawLine(hook->parent,x1+hook->position.x,y1+hook->position.y,
			x2+hook->position.x,y2+hook->position.y);
	RESTORE_GC;
	return err;
}

int GGI_sub_drawvline(ggi_visual *vis,int x,int y,int height)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawVLine(hook->parent,x+hook->position.x,y+hook->position.y,height);
	RESTORE_GC;
	return err;
}

int GGI_sub_drawpixel(ggi_visual *vis,int x,int y)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawPixel(hook->parent,x+hook->position.x,y+hook->position.y);
	RESTORE_GC;
	return err;
}

int GGI_sub_putpixel(ggi_visual *vis,int x,int y,ggi_uint col)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPutPixel(hook->parent,x+hook->position.x,y+hook->position.y,col);
	RESTORE_GC;
	return err;
}

int GGI_sub_getpixel(ggi_visual *vis,int x,int y,ggi_uint *col)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiGetPixel(hook->parent,x+hook->position.x,y+hook->position.y,col);
	RESTORE_GC;
	return err;
}

int GGI_sub_puthline(ggi_visual *vis,int x,int y,int w,void *buffer)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPutHLine(hook->parent,x+hook->position.x,y+hook->position.y,w,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_gethline(ggi_visual *vis,int x,int y,int w,void *buffer)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiGetHLine(hook->parent,x+hook->position.x,y+hook->position.y,w,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_putvline(ggi_visual *vis,int x,int y,int height,void *buffer)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPutVLine(hook->parent,x+hook->position.x,y+hook->position.y,height,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_getvline(ggi_visual *vis,int x,int y,int height,void *buffer)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiGetVLine(hook->parent,x+hook->position.x,y+hook->position.y,height,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_copybox(ggi_visual *vis,int x,int y,int w,int h,int nx,int ny)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiCopyBox(hook->parent,x+hook->position.x,y+hook->position.y,w,h,nx,ny);
	RESTORE_GC;
	return err;
}

/*
**	Simple "call parent" functions
*/

int GGI_sub_setgammamap(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	/* Should we allow that ? */
	return ggiSetGammaMap(hook->parent,start,len,colormap);
}

int GGI_sub_getgammamap(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	return ggiGetGammaMap(hook->parent,start,len,colormap);
}


int GGI_sub_getgamma(ggi_visual *vis,ggi_float *r,ggi_float *g,ggi_float *b)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	return ggiGetGamma(hook->parent,r,g,b);
}

int GGI_sub_setgamma(ggi_visual *vis,ggi_float r,ggi_float g,ggi_float b)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	/* Should we allow that ? */
	return ggiSetGamma(hook->parent,r,g,b);
} 

ggi_pixel GGI_sub_mapcolor(ggi_visual *vis,ggi_color *col)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	return ggiMapColor(hook->parent,col);
}

int GGI_sub_unmappixel(ggi_visual *vis,ggi_uint pixel,ggi_color *col)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	return ggiUnmapPixel(hook->parent,pixel,col);
}

int GGI_sub_setpalvec(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	/* Should we allow that ? */
	return ggiSetPalette(hook->parent,start,len,colormap);
}

int GGI_sub_getpalvec(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	return ggiGetPalette(hook->parent,start,len,colormap);
}

==> libggi-2.0b1.tar.gz: libggi-2.0b1/display/sub/stubs.c <==
/* $Id: stubs.c,v 1.13 1998/12/18 21:36:23 marcus Exp $
******************************************************************************

   Display-sub: stubs

   Copyright (C) 1998 Andreas Beck    [becka@ggi-project.org]
   Copyright (C) 1998 Rudolphe Ortalo

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

/* FIXME: Should we adjust parent clipping for "get" calls ? */

#include <ggi/display/sub.h>

#define IS_SAVING_GC	ggi_pixel _foreground, _background; \
			int _l,_t,_r,_b;

#define SETUP_AND_SAVE_GC \
		do {	_foreground = LIBGGI_GC(hook->parent)->fg_color; \
			_background = LIBGGI_GC(hook->parent)->bg_color; \
			_l = LIBGGI_GC(hook->parent)->cliptl.x; \
			_t = LIBGGI_GC(hook->parent)->cliptl.y; \
			_r = LIBGGI_GC(hook->parent)->clipbr.x; \
			_b = LIBGGI_GC(hook->parent)->clipbr.y; \
			LIBGGI_GC(hook->parent)->fg_color = LIBGGI_GC(vis)->fg_color; \
			LIBGGI_GC(hook->parent)->bg_color = LIBGGI_GC(vis)->bg_color; \
			LIBGGI_GC(hook->parent)->cliptl.x=LIBGGI_GC(vis)->cliptl.x+hook->position.x; \
		   	LIBGGI_GC(hook->parent)->cliptl.y=LIBGGI_GC(vis)->cliptl.y+hook->position.y; \
		   	LIBGGI_GC(hook->parent)->clipbr.x=LIBGGI_GC(vis)->clipbr.x+hook->position.x; \
		   	LIBGGI_GC(hook->parent)->clipbr.y=LIBGGI_GC(vis)->clipbr.y+hook->position.y; \
			/* check limits... (in case leaving visual) */ \
			if (LIBGGI_GC(hook->parent)->clipbr.x > hook->botright.x) \
				LIBGGI_GC(hook->parent)->clipbr.x = hook->botright.x; \
			if (LIBGGI_GC(hook->parent)->clipbr.y > hook->botright.y) \
				LIBGGI_GC(hook->parent)->clipbr.y = hook->botright.y; \
			/* ensure update in KGI */ \
			LIBGGI_GC(hook->parent)->version++; \
		} while (0);

#define RESTORE_GC \
		do {	LIBGGI_GC(hook->parent)->fg_color = _foreground; \
			LIBGGI_GC(hook->parent)->bg_color = _background; \
			LIBGGI_GC(hook->parent)->cliptl.x= _l; \
		   	LIBGGI_GC(hook->parent)->cliptl.y= _t; \
		   	LIBGGI_GC(hook->parent)->clipbr.x= _r; \
		   	LIBGGI_GC(hook->parent)->clipbr.y= _b; \
			/* ensure update in KGI */ \
			LIBGGI_GC(hook->parent)->version++; \
		} while (0);


/*
**	Functions undergoing a translation
**	with GC saving/restoring
*/

/* NB: Here the src is not directly translated... */
int GGI_sub_crossblit(ggi_visual *src, int sx, int sy, int w, int h,
		 ggi_visual *vis, int dx, int dy)
{
	SubHook *hook=LIBGGI_PRIVATE(vis); // Destination is subvisual
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiCrossBlit(src, sx, sy, w, h, hook->parent,
		      dx+hook->position.x, dy+hook->position.y);
	RESTORE_GC;
	return err;
}

int GGI_sub_drawbox(ggi_visual *vis,int x,int y,int width,int length)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawBox(hook->parent,x+hook->position.x,y+hook->position.y,width,length);
	RESTORE_GC;
	return err;
}

int GGI_sub_putbox(ggi_visual *vis,int x,int y,int width,int length,void *buffer)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPutBox(hook->parent,x+hook->position.x,y+hook->position.y,width,length,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_getbox(ggi_visual *vis,int x,int y,int width,int length,void *buffer)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiGetBox(hook->parent,x+hook->position.x,y+hook->position.y,width,length,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_fillscreen(ggi_visual *vis)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawBox(hook->parent, hook->position.x,hook->position.y,
					hook->botright.x-hook->position.x,
					hook->botright.y-hook->position.y);
	RESTORE_GC;
	return err;
}

int GGI_sub_putc(ggi_visual *vis,int x,int y,char c)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPutc(hook->parent,x+hook->position.x,y+hook->position.y,c);
	RESTORE_GC;
	return err;
}


int GGI_sub_puts(ggi_visual *vis,int x,int y,const char *str)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPuts(hook->parent,x+hook->position.x,y+hook->position.y,str);
	RESTORE_GC;
	return err;
}

int GGI_sub_drawhline(ggi_visual *vis,int x,int y,int w)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawHLine(hook->parent,x+hook->position.x,y+hook->position.y,w);
	RESTORE_GC;
	return err;
}

int GGI_sub_drawline(ggi_visual *vis,int x1,int y1,int x2,int y2)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawLine(hook->parent,x1+hook->position.x,y1+hook->position.y,
			x2+hook->position.x,y2+hook->position.y);
	RESTORE_GC;
	return err;
}

int GGI_sub_drawvline(ggi_visual *vis,int x,int y,int height)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawVLine(hook->parent,x+hook->position.x,y+hook->position.y,height);
	RESTORE_GC;
	return err;
}

int GGI_sub_drawpixel(ggi_visual *vis,int x,int y)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiDrawPixel(hook->parent,x+hook->position.x,y+hook->position.y);
	RESTORE_GC;
	return err;
}

int GGI_sub_putpixel(ggi_visual *vis,int x,int y,ggi_uint col)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPutPixel(hook->parent,x+hook->position.x,y+hook->position.y,col);
	RESTORE_GC;
	return err;
}

int GGI_sub_getpixel(ggi_visual *vis,int x,int y,ggi_uint *col)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiGetPixel(hook->parent,x+hook->position.x,y+hook->position.y,col);
	RESTORE_GC;
	return err;
}

int GGI_sub_puthline(ggi_visual *vis,int x,int y,int w,void *buffer)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPutHLine(hook->parent,x+hook->position.x,y+hook->position.y,w,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_gethline(ggi_visual *vis,int x,int y,int w,void *buffer)
{ 
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiGetHLine(hook->parent,x+hook->position.x,y+hook->position.y,w,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_putvline(ggi_visual *vis,int x,int y,int height,void *buffer)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiPutVLine(hook->parent,x+hook->position.x,y+hook->position.y,height,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_getvline(ggi_visual *vis,int x,int y,int height,void *buffer)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiGetVLine(hook->parent,x+hook->position.x,y+hook->position.y,height,buffer);
	RESTORE_GC;
	return err;
}

int GGI_sub_copybox(ggi_visual *vis,int x,int y,int w,int h,int nx,int ny)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);
	int err;
	IS_SAVING_GC;

	SETUP_AND_SAVE_GC;
	err = ggiCopyBox(hook->parent,x+hook->position.x,y+hook->position.y,w,h,nx,ny);
	RESTORE_GC;
	return err;
}

/*
**	Simple "call parent" functions
*/

int GGI_sub_setgammamap(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	/* Should we allow that ? */
	return ggiSetGammaMap(hook->parent,start,len,colormap);
}

int GGI_sub_getgammamap(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	return ggiGetGammaMap(hook->parent,start,len,colormap);
}


int GGI_sub_getgamma(ggi_visual *vis,ggi_float *r,ggi_float *g,ggi_float *b)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	return ggiGetGamma(hook->parent,r,g,b);
}

int GGI_sub_setgamma(ggi_visual *vis,ggi_float r,ggi_float g,ggi_float b)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	/* Should we allow that ? */
	return ggiSetGamma(hook->parent,r,g,b);
} 

ggi_pixel GGI_sub_mapcolor(ggi_visual *vis,ggi_color *col)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	return ggiMapColor(hook->parent,col);
}

int GGI_sub_unmappixel(ggi_visual *vis,ggi_uint pixel,ggi_color *col)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	return ggiUnmapPixel(hook->parent,pixel,col);
}

int GGI_sub_setpalvec(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	/* Should we allow that ? */
	return ggiSetPalette(hook->parent,start,len,colormap);
}

int GGI_sub_getpalvec(ggi_visual *vis,int start,int len,ggi_color *colormap)
{
	SubHook *hook=LIBGGI_PRIVATE(vis);

	return ggiGetPalette(hook->parent,start,len,colormap);
}
